from typing import Optional


def selectOptionQuestion(
    question: str, min: int, max: int, null_possible: Optional[bool] = False
) -> int | None:
    """
    Asks the user to select between a range of values set between min and max\n
    @param: `question`: The question to ask the user.\n
    @param: `min`: The min value\n
    @param: `max`: The max value\n
    @param: `null_posibble`: (Optional) If set to True, users can input empty string instead of the options.\n
    @return: The selected option as int, or if null_possible is set to True, then can also return None.
    """
    invalid_input = True
    while invalid_input:
        user_input = input(f"{question} (select between {min}-{max}):")
        return_value = None
        if user_input == "" and null_possible:
            invalid_input = False
        elif inputIsInt(user_input):
            if min <= int(user_input) <= max:
                invalid_input = False
                return_value = int(user_input)
        else:
            print(f"\x1B[31mInvalid input! Please input within\x1B[37m {min}-{max}")
    return return_value


def inputIsInt(input: str) -> bool:
    try:
        val = int(input)
        return True
    except ValueError:
        try:
            val = float(input)
            return False
        except ValueError:
            return False



from pypeepa.fileInteraction.readJSON import readJSON  # check
from pypeepa.utils.loggingHandler import loggingHandler  # check
from pypeepa.fileInteraction.createDirectory import createDirectory
from typing import Optional, Iterable, Any
from logging import Logger
import json
import os


class ProgressSaver:
    """
    Saves any kind of state in an array.\n
    @init\n
        @param: `app_name`: Name of the file where the progress states will be saved in.\n
    @func: `initialiseJSONSaver`: Initialise the saver before using.\n
    @func: `saveToJSON`: Append the progress state to an array containing previous states and save to `app_name`.save.json\n
        @param: `new_data`: Data to save.\n
        @param: `name`: (Optional)If the save state has any name, it will be used for logging.\n
        @param: `logger` (Optional)A logger object to enable logging.\n
    @func `resetSavedData`: Resets the progress\n
        @param: `logger` (Optional)A logger object to enable logging.\n
    """

    def __init__(self, app_name) -> None:
        self.save_directory = f"{app_name}_saves"
        self.save_file_name = os.path.join(self.save_directory, f"{app_name}.save.json")
        self.saved_data: Optional[Iterable[Any]] = None

    def initialiseJSONSaver(self):
        if not os.path.exists(
            self.save_file_name
        ):  # If save file doesnt exist return empty array
            return []
        self.saved_data = readJSON(self.save_file_name)
        return self.saved_data

    def saveToJSON(
        self, new_data: Any, name: str = None, logger: Optional[Logger] = None
    ):
        self.saved_data.append(new_data)
        createDirectory(self.save_directory)  # Create directory if it doesnt exist
        with open(self.save_file_name, "w+") as completed_output:
            loggingHandler(
                logger, f"Saving file: -> {name}" if name else "Saving file!"
            )
            json.dump(self.saved_data, completed_output)

    def resetSavedData(self, logger: Optional[Logger] = None):
        self.saved_data = []
        with open(self.save_file_name, "w+") as completed_output:
            loggingHandler(logger, f"Clearing saved progress!")
            json.dump(self.saved_data, completed_output)



import pandas as pd
from typing import Callable, Optional, TypeVar, Any, Dict
from pypeepa.checks.checkIfListIsAllNone import checkIfListIsAllNone
from pypeepa.userInteraction.progressBarIterator import progressBarIterator

ArgsType = TypeVar("ArgsType", bound=Dict)


def processCSVInChunks(
    csv_file: str,
    process_function: Callable[[pd.DataFrame, Any], Optional[pd.DataFrame]],
    pf_args: ArgsType,
    chunk_size: Optional[int] = 10000,
    hide_progress_bar: Optional[bool] = False,
):
    """
    Process any CSV file in chunks instead of whole file at once\n\n
    @param: `csv_file`: Path to the csv file.\n
    @param:`process_function`: The function containing the main processing you want to get done.\n
    @param:`pf_args`: Arguments for the function in a dict eg:-\n
                def deleteRowsInCSV(df,`pf_args`):\n
                    # ...delete rows from csv.\n
                    df=df.drop(`pf_args["delete_rows"]`)\n
                    return df\n
                processCSVInChunks("test.csv", deleteRowsInCSV, `{"delete_rows":range(1,20)}`)\n
    @param:`chunk_size`: (Optional) Size of chunks to work with\n
    @param:`hide_progress_bar`: (Optional) Set to True if you dont want the progress bar that comes with this\n
    @return:
        If the process_function has return values than return the values in a list else return None\n
    """
    # Create a generator to read the CSV file in chunks
    chunk_reader = pd.read_csv(
        csv_file, chunksize=chunk_size, low_memory=False, encoding_errors="ignore"
    )

    # Process each chunk and concatenate the results
    processed_chunks = []
    # Show progress bar by default
    # Count total number of lines in the csv_file
    total_chunks = int(
        sum(1 for row in open(csv_file, "r", encoding="cp437", errors="ignore"))
        / chunk_size
    )
    for chunk in (
        chunk_reader
        if hide_progress_bar
        else progressBarIterator(chunk_reader, total_chunks, "Processing file -> ")
    ):
        processed_chunk = process_function(chunk, pf_args)
        processed_chunks.append(processed_chunk)

    if checkIfListIsAllNone(processed_chunks):
        return None

    # Concatenate the processed chunks into a single DataFrame
    df = pd.concat(processed_chunks, ignore_index=True)
    return df
